"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioInput = void 0;
const stream_1 = require("stream");
const os_1 = require("os");
const ChangeVolume_1 = require("../Utils/ChangeVolume");
const ChangeSampleParams_1 = require("../Utils/ChangeSampleParams");
class AudioInput extends stream_1.Writable {
    sampleRate;
    channels;
    volume;
    bitDepth;
    endianness;
    audioMixerArgs;
    removeSelf;
    audioBuffer = Buffer.alloc(0);
    inputClosed = false;
    constructor(args, mixerArgs, removeFunction) {
        super();
        const defaultAudioMixerArgs = {
            sampleRate: 48000,
            channels: 1,
            bitDepth: 16,
            endianness: (0, os_1.endianness)()
        };
        this.sampleRate = args.sampleRate ?? 48000;
        this.channels = args.channels ?? 1;
        this.volume = args.volume ?? 100;
        this.bitDepth = args.bitDepth ?? 16;
        this.endianness = args.endianness ?? (0, os_1.endianness)();
        this.audioMixerArgs = mixerArgs || defaultAudioMixerArgs;
        this.removeSelf = removeFunction || null;
        this.once("close", () => {
            this.inputClosed = true;
            if (this.audioBuffer.length > 0)
                return;
            this.removeSelf(this);
        });
    }
    _write(chunk, _, callback) {
        if (this.inputClosed)
            return;
        const inputParams = {
            sampleRate: this.sampleRate,
            channels: this.channels,
            bitDepth: this.bitDepth,
            endianness: this.endianness
        };
        const processedChunk = (0, ChangeSampleParams_1.default)(chunk, inputParams, this.audioMixerArgs);
        this.audioBuffer = Buffer.concat([this.audioBuffer, processedChunk]);
        callback();
    }
    setVolume(volume) {
        this.volume = volume;
        return this;
    }
    get availableAudioLength() {
        return this.audioBuffer?.length;
    }
    close() {
        this.inputClosed = true;
        if (this.audioBuffer.length === 0)
            this.removeSelf(this);
        this.end();
    }
    readAudioChunk(highWaterMark) {
        if (this.audioBuffer.length === 0) {
            if (this.inputClosed && this.removeSelf)
                this.removeSelf(this);
            return this.audioBuffer;
        }
        const changeVolumeArgs = {
            volume: this.volume,
            bitDepth: this.audioMixerArgs.bitDepth,
            endianness: this.audioMixerArgs.endianness
        };
        let chunk = Buffer.alloc(0);
        chunk = this.audioBuffer.subarray(0, highWaterMark);
        this.audioBuffer = this.audioBuffer.subarray(highWaterMark, this.audioBuffer.length);
        if (this.audioBuffer.length < highWaterMark) {
            const bytesPerSample = this.audioMixerArgs.bitDepth / 8;
            const silentChunkLength = (highWaterMark - chunk.length) * bytesPerSample;
            const silentChunk = Buffer.alloc(silentChunkLength);
            chunk = Buffer.concat([chunk, silentChunk]);
        }
        return (0, ChangeVolume_1.default)(chunk, changeVolumeArgs);
    }
}
exports.AudioInput = AudioInput;
