"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioMixer = void 0;
const stream_1 = require("stream");
const os_1 = require("os");
const AudioInput_1 = require("../AudioInput/AudioInput");
const mixAudioChunks_1 = require("../Utils/mixAudioChunks");
const ChangeVolume_1 = require("../Utils/ChangeVolume");
class AudioMixer extends stream_1.Readable {
    sampleRate;
    channels;
    volume;
    bitDepth;
    endianness;
    highWaterMark;
    delayTime;
    autoClose;
    inputs = [];
    mixerClosed = false;
    constructor(args) {
        super();
        this.sampleRate = args.sampleRate ?? 48000;
        this.channels = args.channels ?? 1;
        this.volume = args.volume ?? 100;
        this.bitDepth = args.bitDepth ?? 16;
        this.endianness = args.endianness ?? (0, os_1.endianness)();
        this.highWaterMark = args.highWaterMark ?? null;
        this.delayTime = args.delayTime ?? 1;
        this.autoClose = args.autoClose ?? false;
        this.loopRead();
        this.once("close", this.close);
    }
    _read() {
        let chunk = Buffer.alloc(0);
        if (this.inputs.length === 0)
            return;
        const chunks = [];
        const mixerArgs = {
            sampleRate: this.sampleRate,
            channels: this.channels,
            volume: this.volume,
            bitDepth: this.bitDepth,
            endianness: this.endianness,
            highWaterMark: this.highWaterMark
        };
        const chunkSize = this.highWaterMark || Math.min(...this.inputs.map(input => input.availableAudioLength || 0));
        this.inputs.forEach(async (i) => chunks.push(i.readAudioChunk(chunkSize)));
        chunk = (0, mixAudioChunks_1.default)(chunks, chunkSize, mixerArgs);
        const changeVolumeArgs = {
            volume: this.volume,
            bitDepth: this.bitDepth,
            endianness: this.endianness
        };
        this.unshift((0, ChangeVolume_1.default)(chunk, changeVolumeArgs));
    }
    setVolume(volume) {
        this.volume = volume;
        return this;
    }
    createAudioInput(inputArgs) {
        const mixerArgs = {
            sampleRate: this.sampleRate,
            channels: this.channels,
            bitDepth: this.bitDepth,
            endianness: this.endianness,
        };
        const audioInput = new AudioInput_1.AudioInput(inputArgs, mixerArgs, this.removeAudioInput.bind(this));
        this.inputs.push(audioInput);
        this.emit("addInput");
        return audioInput;
    }
    removeAudioInput(audioInput) {
        const findAudioInput = this.inputs.indexOf(audioInput);
        if (findAudioInput !== -1) {
            this.inputs.splice(findAudioInput, 1);
            this.emit("removeInput");
        }
        if (this.inputs.length === 0 && this.autoClose)
            return this.close();
        return this;
    }
    close() {
        if (this.mixerClosed && this.inputs.length === 0)
            return this;
        this.inputs.forEach(input => input.close());
        this.destroy();
        this.unshift(null);
        this.mixerClosed = true;
        return this;
    }
    loopRead() {
        if (this.mixerClosed)
            return;
        this._read();
        const delayTime = typeof this.delayTime === "number" ? this.delayTime : this.delayTime();
        setTimeout(this.loopRead.bind(this), delayTime);
    }
}
exports.AudioMixer = AudioMixer;
